@@ -8,23 +8,23 @@
     :copyright: Conceptual Vision Consulting LLC 2018-2019, see AUTHORS for more details.
     :license: MIT, see LICENSE for more details.
 """
- 
-import threading
+from typing import Optional, Any, List, TypeVar
 
-from pip_services3_commons.refer import IReferenceable
-from pip_services3_commons.run import IOpenable, IClosable, ICleanable
-from pip_services3_components.log import CompositeLogger
-from pip_services3_commons.data import IdGenerator
-from ..IWriter import IWriter
+from pip_services3_commons.data import IdGenerator, IIdentifiable, AnyValueMap
+
+from pip_services3_data import ISaver, ILoader
+from .MemoryPersistence import MemoryPersistence
 from ..IGetter import IGetter
 from ..ISetter import ISetter
-
-from .MemoryPersistence import MemoryPersistence
+from ..IWriter import IWriter
 
 # This function will be overriden in the code
 filtered = filter
 
-class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter):
+T = TypeVar('T')  # Declare type variable
+
+
+class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter, IIdentifiable):
     """
     Abstract persistence component that stores data in memory
     and implements a number of CRUD operations over data items
@@ -32,24 +32,25 @@ class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter
     interface.
 
     In basic scenarios child classes shall only override
-    [[getPageByFilter]], [[getListByFilter]] or [[deleteByFilter]]
+    :func:`get_page_by_filter`, :func:`get_list_by_filter` or :func:`delete_by_filter`
     operations with specific filter function. All other operations
     can be used out of the box.
 
     In complex scenarios child classes can implement additional
     operations by accessing cached items via this._items property
-    and calling [[save]] method on updates.
+    and calling :func:`save` method on updates.
 
     ### Configuration parameters ###
-
         - options:
             - max_page_size:       Maximum number of items returned in a single page (default: 100)
 
     ### References ###
-
-        - *:logger:*:*:1.0       (optional) ILogger components to pass log messages
+        - `*:logger:*:*:1.0`       (optional) ILogger components to pass log messages
 
     Example:
+
+    .. code-block:: python
+
         class MyMemoryPersistence(IdentifiableMemoryPersistence):
 
             def get_page_by_filter(self, correlationId, filter, paging):
@@ -63,11 +64,9 @@ class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter
             print str(mydata.get_data())
 
             persistence.delete_by_id("123", "1")
-            ...
     """
-    _max_page_size = 100
 
-    def __init__(self, loader = None, saver = None):
+    def __init__(self, loader: ILoader = None, saver: ISaver = None):
         """
         Creates a new instance of the persistence.
 
@@ -77,8 +76,13 @@ class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter
         """
         super(IdentifiableMemoryPersistence, self).__init__(loader, saver)
 
+    def __convert_to_obj(self, item):
+        if isinstance(item, dict):
+            item = type('object', (object,), item)
 
-    def get_list_by_ids(self, correlation_id, ids):
+        return item
+
+    def get_list_by_ids(self, correlation_id: Optional[str], ids: List[Any]) -> List[T]:
         """
         Gets a list of data items retrieved by given unique ids.
 
@@ -88,20 +92,20 @@ class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter
 
         :return: a data list of results by ids.
         """
+
         def filter(item):
-            return item['id'] in ids
+            item = self.__convert_to_obj(item)
+            return item.id in ids
 
         return self.get_list_by_filter(correlation_id, filter)
 
-
     def _find_one(self, id: str):
         for item in self._items:
-            if item['id'] == id:
+            if item.id == id:
                 return item
         return None
 
-
-    def get_one_by_id(self, correlation_id: str, id: str):
+    def get_one_by_id(self, correlation_id: Optional[str], id: Any) -> T:
         """
         Gets a data item by its unique id.
 
@@ -123,8 +127,7 @@ class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter
             self._logger.trace(correlation_id, "Cannot find item by " + str(id))
         return item
 
-
-    def create(self, correlation_id, item):
+    def create(self, correlation_id: Optional[str], item: T) -> T:
         """
         Creates a data item.
 
@@ -134,13 +137,14 @@ class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter
 
         :return: a created item
         """
-        if 'id' not in item or item['id'] is None:
-            item['id'] = IdGenerator.next_long()
+        item = self.__convert_to_obj(item)
+
+        if not hasattr(item, 'id') or item.id is None:
+            item.id = IdGenerator.next_long()
 
         return super().create(correlation_id, item)
 
-
-    def set(self, correlation_id, item):
+    def set(self, correlation_id: Optional[str], item: T) -> T:
         """
         Sets a data item. If the data item exists it updates it, otherwise it create a new data item.
 
@@ -150,13 +154,15 @@ class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter
 
         :return: an updated item
         """
-        if 'id' not in item or item['id'] == None:
-            item['id'] = IdGenerator.next_long()
+        item = self.__convert_to_obj(item)
+
+        if not hasattr(item, 'id') or item.id is None:
+            item.id = IdGenerator.next_long()
 
         self._lock.acquire()
         try:
-            old_item = self._find_one(item['id'])
-            if old_item == None:
+            old_item = self._find_one(item.id)
+            if old_item is None:
                 self._items.append(item)
             else:
                 index = self._items.index(old_item)
@@ -173,8 +179,7 @@ class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter
         self.save(correlation_id)
         return item
 
-
-    def update(self, correlation_id, new_item):
+    def update(self, correlation_id: Optional[str], new_item: T) -> T:
         """
         Updates a data item.
 
@@ -186,10 +191,12 @@ class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter
         """
         self._lock.acquire()
         try:
-            old_item = self._find_one(new_item['id'])
-            if old_item == None:
+            new_item = self.__convert_to_obj(new_item)
+
+            old_item = self._find_one(new_item.id)
+            if old_item is None:
                 return None
-            
+
             index = self._items.index(old_item)
             if index < 0: return None
 
@@ -203,7 +210,7 @@ class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter
         self.save(correlation_id)
         return new_item
 
-    def update_partially(self, correlation_id, id, data):
+    def update_partially(self, correlation_id: Optional[str], id: Any, data: AnyValueMap) -> T:
         """
         Updates only few selected fields in a data item.
 
@@ -220,11 +227,11 @@ class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter
         self._lock.acquire()
         try:
             old_item = self._find_one(id)
-            if old_item == None:
+            if old_item is None:
                 return None
-            
-            for (k, v) in data.items():
-                old_item[k] = v
+
+            for k, v in data.items():
+                setattr(old_item, k, v)
 
             new_item = old_item
         finally:
@@ -236,8 +243,7 @@ class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter
         self.save(correlation_id)
         return new_item
 
-
-    def delete_by_id(self, correlation_id, id):
+    def delete_by_id(self, correlation_id: Optional[str], id: Any) -> T:
         """
         Deleted a data item by it's unique id.
 
@@ -250,8 +256,8 @@ class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter
         self._lock.acquire()
         try:
             item = self._find_one(id)
-            if item == None: return None
-            
+            if item is None: return None
+
             index = self._items.index(item)
             if index < 0: return None
 
@@ -264,8 +270,7 @@ class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter
         self.save(correlation_id)
         return item
 
-
-    def delete_by_ids(self, correlation_id, ids):
+    def delete_by_ids(self, correlation_id: Optional[str], ids: List[Any]):
         """
         Deletes multiple data items by their unique ids.
 
@@ -273,7 +278,10 @@ class IdentifiableMemoryPersistence(MemoryPersistence, IWriter, IGetter, ISetter
 
         :param ids: ids of data items to be deleted.
         """
+
         def filter(item):
-            return item['id'] in ids
-        
-        self.delete_by_filter(correlation_id, filter)
\ No newline at end of file
+            item = self.__convert_to_obj(item)
+
+            return item.id in ids
+
+        self.delete_by_filter(correlation_id, filter)
